﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Char;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Integer;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.IntegerVector;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.String;

  public class WordStats : BenchmarkSuiteDataDescriptor {
    private const string name = "Word Stats - Hard";
    private const string fileName = "WordStats.csv";
    private const string description = "Given a string, print the number of words containing n characters for n from 1 to the length of the longest word. At the end of the output, print a line that gives the number of sentences and line that gives the average sentence length. A word is any string of consecutive non-whitespace characters(including sentence terminators). Every string will contain at least one sentence terminator(period, exclamation point, or question mark). The average sentence length is the number of words in the file divided by the number of sentence terminator characters.";

    protected override string FileName { get { return fileName; } }
    public override string Name { get { return name; } }
    public override string Description { get { return description; } }
    protected override int InputArgumentCount { get { return 1; } }
    protected override int OutputArgumentCount { get { return 1; } }

    public override ProblemData CreateProblemData() {
      return new ProblemData {
        Name = Name,
        Description = Description,
        Examples = CloneExamples(),
        BestResult = 0,
        WorstResult = 1000,
        InputArgumentTypes = new[] { ExampleArgumentType.String },
        OutputArgumentTypes = new[] { ExampleArgumentType.Print },
        TrainingCount = 100,
        TestCount = 1000,
        EnabledDataTypes = DataTypes.Exec | DataTypes.Integer | DataTypes.Float | DataTypes.Boolean | DataTypes.Char | DataTypes.String | DataTypes.IntegerVector | DataTypes.FloatVector | DataTypes.StringVector | DataTypes.Print,
        MaxSize = 1000,
        EvalLimit = 6000,
        ErcOptions = {
          ErcProbability = 0.05,
          IntegerErcOptions = new IntegerErcOptions(
            new IntegerRangeErc(-100, 100)),
          IntegerVectorErcOptions = new IntegerVectorErcOptions(
            new IntegerVectorConstantsErc(new int[0])),
          CharErcOptions = new CharErcOptions(
            new IntegerConstantErc('.', '?', '!', ' ', '\t', '\n', ':')),
          StringErcOptions = new StringErcOptions(
            new StringConstantErc("words of length ", ": ", "number of sentences: ", "average sentence length: "))
        }
      };
    }

    protected override Example ParseExample(string[] input, string[] output) {
      return new Example {
        InputArgs = input,
        OutputArgs = output,
        InputString = input,
        OutputPrint = output[0]
      };
    }
  }
}
