﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System;

  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the subvec of the top item on the type stack.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  [StorableClass]
  public abstract class VectorSubExpression<T> : StatelessExpression {
    protected VectorSubExpression() { }
    [StorableConstructor]
    protected VectorSubExpression(bool deserializing) : base(deserializing) { }

    protected bool IsNoop(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      return vectorStack.IsEmpty || interpreter.IntegerStack.Count < 2;
    }

    protected void Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      var first = (int)Math.Min(vectorStack.Top.Count, Math.Max(0, interpreter.IntegerStack[1]));
      var second = (int)Math.Min(vectorStack.Top.Count, Math.Max(first, interpreter.IntegerStack.Top));
      interpreter.IntegerStack.Remove(2);

      if (vectorStack.Top.Count == 0)
        return;

      var result = vectorStack.Top.GetRange(first, second - first);
      vectorStack.SetTop(result);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.IntegerVector, "INTEGER[].SUB", StackTypes.Integer)]
  public class IntegerVectorSubExpression : VectorSubExpression<long> {
    public IntegerVectorSubExpression() { }
    [StorableConstructor]
    protected IntegerVectorSubExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter, interpreter.IntegerVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.FloatVector, "FLOAT[].SUB", StackTypes.Integer)]
  public class FloatVectorSubExpression : VectorSubExpression<double> {
    public FloatVectorSubExpression() { }
    [StorableConstructor]
    protected FloatVectorSubExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter, interpreter.FloatVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].SUB", StackTypes.Integer)]
  public class BooleanVectorSubExpression : VectorSubExpression<bool> {
    public BooleanVectorSubExpression() { }
    [StorableConstructor]
    protected BooleanVectorSubExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter, interpreter.BooleanVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [StorableClass]
  [PushExpression(StackTypes.StringVector, "STRING[].SUB", StackTypes.Integer)]
  public class StringVectorSubExpression : VectorSubExpression<string> {
    public StringVectorSubExpression() { }
    [StorableConstructor]
    protected StringVectorSubExpression(bool deserializing) : base(deserializing) { }

    public override bool IsNoop(IInternalPushInterpreter interpreter) {
      return IsNoop(interpreter, interpreter.StringVectorStack);
    }

    public override void Eval(IInternalPushInterpreter interpreter) {
      Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}