﻿namespace HeuristicLab.Tests.Benchmark {
  using System.Collections.Generic;
  using System.Linq;

  using HeuristicLab.BenchmarkSuite;
  using HeuristicLab.BenchmarkSuite.Problems;

  using Microsoft.VisualStudio.TestTools.UnitTesting;

  public class DataTests {
    public void DataLoadedTest(BenchmarkSuiteDataDescriptor descriptor) {
      var data = descriptor.CreateProblemData();

      foreach (var example in data.Examples) {
        Assert.AreEqual(example.InputArgs.Length + example.OutputArgs.Length, data.TotalArgumentCount);
        var inputTypeCount = GetTypeCounterDict(data.InputArgumentTypes);
        var outputTypeCount = GetTypeCounterDict(data.OutputArgumentTypes);

        for (var i = 0; i < data.InputArgumentTypes.Length; i++)
          TestPerArgumentType(i, example, data.InputArgumentTypes[i], inputTypeCount);

        //for (var i = 0; i < data.OutputArgumentTypes.Length; i++)
        //  TestPerArgumentType(example.OutputArgs[i], example, data.OutputArgumentTypes[i], outputTypeCount);
      }
    }

    private IReadOnlyDictionary<ExampleArgumentType, ArgumentTypeCounter> GetTypeCounterDict(
      ExampleArgumentType[] types) {
      return types
        .GroupBy(type => type)
        .ToDictionary(group => group.Key, group => new ArgumentTypeCounter(group.Count()));
    }

    private void TestPerArgumentType(
      int i,
      Example example,
      ExampleArgumentType type,
      IReadOnlyDictionary<ExampleArgumentType, ArgumentTypeCounter> inputTypeCount) {

      inputTypeCount[type].Counter++;
      Assert.IsTrue(inputTypeCount[type].Counter < inputTypeCount[type].Total);
    }

    private class ArgumentTypeCounter {
      public ArgumentTypeCounter(int total, int counter = 0) {
        Total = total;
        Counter = counter;
      }
      public int Total { get; set; }
      public int Counter { get; set; }
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Checksum() {
      DataLoadedTest(new Checksum());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void CollatzNumbers() {
      DataLoadedTest(new CollatzNumbers());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void CompareStringLengths() {
      DataLoadedTest(new CompareStringLengths());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void CountOdds() {
      DataLoadedTest(new CountOdds());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Digits() {
      DataLoadedTest(new Digits());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void DoubleLetters() {
      DataLoadedTest(new DoubleLetters());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void EvenSquares() {
      DataLoadedTest(new EvenSquares());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void ForLoopIndex() {
      DataLoadedTest(new ForLoopIndex());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Grades() {
      DataLoadedTest(new Grades());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void LastIndexOfZero() {
      DataLoadedTest(new LastIndexOfZero());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Median() {
      DataLoadedTest(new Median());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void MirrorImage() {
      DataLoadedTest(new MirrorImage());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void NegativeToZero() {
      DataLoadedTest(new NegativeToZero());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void NumberIo() {
      DataLoadedTest(new NumberIO());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void PigLatin() {
      DataLoadedTest(new PigLatin());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void ReplaceSpaceWithNewLine() {
      DataLoadedTest(new ReplaceSpaceWithNewline());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void ScrabbleScore() {
      DataLoadedTest(new ScrabbleScore());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Smallest() {
      DataLoadedTest(new Smallest());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void SmallOrLarge() {
      DataLoadedTest(new SmallOrLarge());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void StringDifferences() {
      DataLoadedTest(new StringDifferences());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void StringLengthsBackwards() {
      DataLoadedTest(new StringLengthsBackwards());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void SumOfSquares() {
      DataLoadedTest(new SumOfSquares());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void SuperAnagrams() {
      DataLoadedTest(new SuperAnagrams());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void Syllables() {
      DataLoadedTest(new Syllables());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void VectorAverage() {
      DataLoadedTest(new VectorAverage());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void VectorSummed() {
      DataLoadedTest(new VectorSummed());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void WallisPi() {
      DataLoadedTest(new WallisPi());
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void WordStats() {
      DataLoadedTest(new WordStats());
    }
  }
}
